﻿import { CheckBoxComponent } from './components/checkbox/checkbox-component';
import { FreeTextData } from './models/DataField/FreeTextData';
import { BooleanDecisionComponent } from './components/booleanDecision/booleanDecision-component';
import { FreeTextComponent } from './components/freetext/freetext-component';
import { TextAreaComponent } from './components/textarea/textarea-component';
import { RadioBoxComponent } from './components/radiobox/radiobox-component';
import { WordlistComponent } from './components/wordlist/wordlist-component';
import { AutocompleteComponent } from './components/autocomplete/autocomplete-component';
import { ComboboxComponent } from './components/combobox/combobox-component';


import { BrowserModule } from '@angular/platform-browser';
import { RouterModule, Routes } from '@angular/router';
import { NgModule } from '@angular/core';
import { HttpClientModule, HttpClient, HTTP_INTERCEPTORS } from '@angular/common/http';
import { HttpModule } from '@angular/http';
import { DataTableModule } from "angular2-datatable";
import { NgbModule } from '@ng-bootstrap/ng-bootstrap';
import { DialogService } from "ng2-bootstrap-modal";
import { ReactiveFormsModule } from '@angular/forms';
import { ContextMenuModule } from 'ngx-contextmenu';
import { Ng2Webstorage } from 'ngx-webstorage';

import { RestBase } from './services/rest-base';
import { TokenService } from './services/login/token.service';
import { NativeLoginService } from './services/login/native-login.service';
import { DatasetsServiceService } from './services/datasets-service.service';
import { DatasetProfileService } from './services/dataset-profile.service';
import { DmpsServiceService } from './services/dmps-service.service';
import { ProjectService } from './services/project-service';
import { ServiceService } from './services/service-service';
import { ResearcherService } from './services/researcher-service';
import { OrganisationService } from './services/organisation-service';
import { DmpProfileService } from './services/dmpprofile-service';
import { RegistryService } from './services/registry-service';
import { DatarepoService } from './services/datarepo-service';
import { GlobalInterceptor } from './services/interceptor';
import { Toolbox } from './services/toolbox';

import { DataFilterPipe } from './pipes/data-filter.pipe';
import { DatasetProfileTableFilterPipe } from './pipes/datasetprofile-table-filter.pipe';
import { DmpTableFilterPipe } from './pipes/dmp-table-filter.pipe';
import { ServiceTableFilterPipe } from './pipes/service-table-filter.pipe';
import { DmpProfileTableFilterPipe } from './pipes/dmp-profile-table-filter.pipe';
import { ProjectTableFilterPipe } from './pipes/project-table-filter.pipe';
import { RegistryTableFilterPipe } from './pipes/registry-table-filter.pipe';
import { OrganisationTableFilterPipe } from './pipes/organisation-table-filter.pipe';
import { ResearcherTableFilterPipe } from './pipes/researcher-table-filter.pipe';
import { DatarepoTableFilterPipe } from './pipes/datarepo-table-filter.pipe';


import { GooggleSignInComponent } from './login/googgle-sign-in/googgle-sign-in.component';
import { MainSignInComponent } from './login/main-sign-in/main-sign-in.component';
import { ProjectEditorComponent } from './managers/project-editor/project-editor.component';
import { DatasetsViewerComponent } from './datasets-viewer/datasets-viewer.component';
import { AppComponent } from './app.component';
import { FormsModule } from '@angular/forms';
import { ProfileEditorComponent } from './bootstrap/profile-editor/profile-editor.component';
import { PropertiesEditorComponent } from './bootstrap/properties-editor/properties-editor.component';
import { NewDatasetComponent } from './bootstrap/new-dataset/new-dataset.component';
import { ConfirmationComponent } from './bootstrap/confirmation/confirmation.component';
import { DmpEditorComponent } from './managers/dmp-editor/dmp-editor.component';
import { MainWindowComponent } from './main-window/main-window.component';
import { AppRouting } from './routes';
import { DmpProfileEditorComponent } from './managers/dmp-profile-editor/dmp-profile-editor.component';
import { OrganisationEditorComponent } from './managers/organisation-editor/organisation-editor.component';
import { RegistryEditorComponent } from './managers/registry-editor/registry-editor.component';
import { ServiceEditorComponent } from './managers/service-editor/service-editor.component';
import { ResearcherEditorComponent } from './managers/researcher-editor/researcher-editor.component';
import { DatareposEditorComponent } from './managers/datarepos-editor/datarepos-editor.component';
import { DatasetprofileEditorComponent } from './managers/datasetprofile-editor/datasetprofile-editor.component';
import { DatasetProfileGUIEditorComponent } from './dataset-profile-gui-editor/dataset-profile-gui-editor.component';

import { FieldFormComponent } from './dataset-profile-form/field-form/field-form.component';
import { FormComponent } from './dataset-profile-form/form/form.component';
import { GroupFieldFormComponent } from './dataset-profile-form/groupfield-form/groupfield-form.component';
import { RuleFormComponent } from './dataset-profile-form/rule-component/rule.component';
import { SectionFormComponent } from './dataset-profile-form/section-form/section-form.component';
import { PageFormComponent } from './dataset-profile-form/page-form/page-component';
import { CompositeFieldFormComponent } from './dataset-profile-form/compositefield-form/compositefield-form.component';




@NgModule({
  declarations: [
    DataFilterPipe,
    DmpTableFilterPipe,
    DmpProfileTableFilterPipe,
    DatasetProfileTableFilterPipe,
    ProjectTableFilterPipe,
    OrganisationTableFilterPipe,
    RegistryTableFilterPipe,
    ServiceTableFilterPipe,
    ResearcherTableFilterPipe,
    DatarepoTableFilterPipe,

    AppComponent,
    GooggleSignInComponent,
    FieldFormComponent,
    FormComponent,
    GroupFieldFormComponent,
    RuleFormComponent,
    SectionFormComponent,
    PageFormComponent,
    CompositeFieldFormComponent,
    ComboboxComponent,
    AutocompleteComponent,
    WordlistComponent,
    RadioBoxComponent,
    DatasetsViewerComponent,
    ProfileEditorComponent,
    PropertiesEditorComponent,
    NewDatasetComponent,
    ConfirmationComponent,
    DmpEditorComponent,
    MainWindowComponent,
    ProjectEditorComponent,
    DmpProfileEditorComponent,
    OrganisationEditorComponent,
    RegistryEditorComponent,
    ServiceEditorComponent,
    MainSignInComponent,
    ResearcherEditorComponent,
    DatareposEditorComponent,
    DatasetprofileEditorComponent,
    DatasetProfileGUIEditorComponent,
    TextAreaComponent,
    CheckBoxComponent,
    BooleanDecisionComponent,
    FreeTextComponent
  ],
  imports: [
    BrowserModule,
    HttpClientModule,
    HttpModule,
    DataTableModule,
    FormsModule,
    ReactiveFormsModule,
    ContextMenuModule,
    Ng2Webstorage,
    NgbModule.forRoot(),
    AppRouting
  ],
  providers: [{
    provide: HTTP_INTERCEPTORS,
    useClass: GlobalInterceptor,
    multi: true,
  },
    DatasetsServiceService, TokenService, DmpsServiceService, ResearcherService,
    DialogService, DatasetProfileService, ProjectService, OrganisationService, DatarepoService,
    DmpProfileService, RegistryService, ServiceService, NativeLoginService, HttpModule, Toolbox, HttpClient, RestBase],
  bootstrap: [AppComponent]
})
export class AppModule { }
