﻿import { BaseModel } from '../BaseModel';
import { FormBuilder, FormGroup, FormControl, Validators } from '@angular/forms';
import { FormGenerator } from '../interfaces/FormGenerator';
import { JsonSerializer } from 'app/utilities/JsonSerializer';
import { Section } from './Section';
import { Serializable } from '../interfaces/Serializable';
import { Page } from "./Page";

export class DatasetProfileModel extends BaseModel implements Serializable<DatasetProfileModel>,FormGenerator<FormGroup>{

	public sections: Array<Section> = new Array<Section>();
	public pages: Array<Page> = new Array<Page>();
    public label: string;

    fromJSONObject(item:any):DatasetProfileModel{
		this.sections = new JsonSerializer<Section>().fromJSONArray(item.sections, Section);
		this.pages = new JsonSerializer<Page>().fromJSONArray(item.pages, Page);
        this.label = item.label;
        return this;
    }

    buildForm():FormGroup{
		let formGroup: FormGroup = new FormBuilder().group({});

        let sectionsFormArray = new Array<FormGroup>();
        this.sections.forEach(item => {
            let form: FormGroup = item.buildForm();
            sectionsFormArray.push(form)
        })
		formGroup.addControl('sections', this.formBuilder.array(sectionsFormArray));

		let pagesFormArray = new Array<FormGroup>();
		// this.pages.forEach(item => {
		// 	let form: FormGroup = item.buildForm();
		// 	pagesFormArray.push(form)
		// })
		formGroup.addControl('pages', this.formBuilder.array(pagesFormArray));

        formGroup.addControl('label', new FormControl(this.label));
        return formGroup;
    }

}