package eu.eudat.controllers;

import java.util.UUID;

import eu.eudat.dao.entities.*;
import eu.eudat.managers.DashBoardManager;
import eu.eudat.models.dashboard.DashBoardStatistics;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import eu.eudat.managers.AdminManager;
import eu.eudat.models.admin.composite.DatasetProfile;

import javax.transaction.Transactional;

@RestController
@CrossOrigin
public class Admin {

	@Autowired private DatasetProfileDao datasetProfileDao;
	@Autowired private DatasetProfileRulesetDao datasetProfileRulesetDao;
	@Autowired private DatasetProfileViewstyleDao datasetProfileViewstyleDao;

	@Transactional
	@RequestMapping(method = RequestMethod.POST, value = { "/admin/addDmp" },consumes = "application/json", produces="application/json")
	public ResponseEntity<Object> addDmp(@RequestBody DatasetProfile profile){
		try{
			eu.eudat.entities.DatasetProfile modelDefinition = AdminManager.generateViewStyleDefinition(profile);
			datasetProfileDao.createOrUpdate(modelDefinition);

			return ResponseEntity.status(HttpStatus.OK).body(modelDefinition.getId());
		}catch(Exception ex){
			ex.printStackTrace();
			return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("{\"reason\":\""+ex.getMessage()+"\"}");
		}
	} 
	
	@Transactional
	@RequestMapping(method = RequestMethod.POST, value = { "/admin/addDmp/{id}" },consumes = "application/json", produces="application/json")
	public ResponseEntity<Object> updateDmp(@PathVariable String id,@RequestBody DatasetProfile profile){
		try{
			eu.eudat.entities.DatasetProfile modelDefinition = AdminManager.generateViewStyleDefinition(profile);

			eu.eudat.entities.DatasetProfile datasetprofile = datasetProfileDao.find(UUID.fromString(id));
			datasetprofile.setDefinition(modelDefinition.getDefinition());
			datasetProfileDao.createOrUpdate(datasetprofile);
			
			return ResponseEntity.status(HttpStatus.OK).body(null);
		}catch(Exception ex){
			ex.printStackTrace();
			return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("{\"reason\":\""+ex.getMessage()+"\"}");
		}
	} 
	
	@RequestMapping(method = RequestMethod.GET, value = { "/admin/get/{id}" }, produces="application/json")
	public ResponseEntity<Object> get(@PathVariable String id){
		try{
			eu.eudat.entities.DatasetProfile profile = datasetProfileDao.find(UUID.fromString(id));
			eu.eudat.models.admin.composite.DatasetProfile datasetprofile = AdminManager.generateDatasetProfileModel(profile);
			return ResponseEntity.status(HttpStatus.OK).body(datasetprofile);
		}catch(Exception ex){
			ex.printStackTrace();
			return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("{\"reason\":\""+ex.getMessage()+"\"}");
		}
	}
}
