package eu.eudat.controllers;

import java.net.HttpURLConnection;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import eu.eudat.models.components.commons.datafield.AutoCompleteData;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import eu.eudat.dao.entities.DatasetDao;
import eu.eudat.dao.entities.DatasetProfileDao;
import eu.eudat.dao.entities.DatasetProfileRulesetDao;
import eu.eudat.dao.entities.DatasetProfileViewstyleDao;
import eu.eudat.managers.UserManager;
import eu.eudat.models.helpers.AutoCompleteLookupItem;
import eu.eudat.models.properties.PropertiesModel;
import eu.eudat.utilities.builders.XmlBuilder;

@RestController
@CrossOrigin
public class DatasetProfileController {
	
	@Autowired private DatasetProfileDao datasetProfileDao;
	@Autowired private DatasetProfileRulesetDao datasetProfileRulesetDao;
	@Autowired private DatasetProfileViewstyleDao datasetProfileViewstyleDao;
	@Autowired private DatasetDao datasetDao;

	@RequestMapping(method = RequestMethod.GET, value = { "/datasetwizard/get/{id}" }, produces="application/json")
	public ResponseEntity<Object> getSingle(@PathVariable String id){
		try {
			eu.eudat.entities.Dataset dataset = datasetDao.find(UUID.fromString(id));
			eu.eudat.models.user.composite.DatasetProfile datasetprofile = UserManager.generateDatasetProfileModel(dataset.getProfile());
			datasetprofile.setStatus(dataset.getStatus());
			if(dataset.getProperties()!=null){
				JSONObject jobject = new JSONObject(dataset.getProperties());
				Map<String,Object> properties = (Map<String, Object>)jobject.toMap();
				datasetprofile.fromJsonObject(properties);				
			}
			return ResponseEntity.status(HttpStatus.OK).body(datasetprofile);
		}
		catch(Exception ex) {
			ex.printStackTrace();
			return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("Serialization issue: "+ex.getMessage());
		}
	} 
	
	
	@RequestMapping(method = RequestMethod.POST, value = { "/datasetprofile/save/{id}" }, consumes="application/json",produces="application/json")
	public ResponseEntity<Object> updateDataset(@PathVariable String id,@RequestBody PropertiesModel properties){
		try {
			eu.eudat.entities.Dataset dataset = datasetDao.find(UUID.fromString(id));
			Map<String,Object> values = new HashMap();
			properties.toMap(values);
			JSONObject jobject = new JSONObject(values);
			dataset.setProperties(jobject.toString());
			dataset.setStatus((short)properties.getStatus());
			datasetDao.createOrUpdate(dataset);  //TODO

			return ResponseEntity.status(HttpStatus.OK).body(properties);
		}
		catch(Exception ex) {
			return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("Serialization issue: "+ex.getMessage());
		}
	} 
	
	@RequestMapping(method = RequestMethod.POST, value = { "/search/autocomplete" }, consumes="application/json",produces="application/json")
	public ResponseEntity<Object> getDataForAutocomplete(@RequestBody AutoCompleteLookupItem lookupItem){
		try {
			eu.eudat.entities.Dataset dataset = datasetDao.find(UUID.fromString(lookupItem.getProfileID()));
			//Document viewStyleDoc = XmlBuilder.fromXml(dataset.getProfile().getViewstyle().getDefinition());
			//Element field = viewStyleDoc.getElementById(lookupItem.getFieldID());
			eu.eudat.entities.xmlmodels.datasetprofiledefinition.Field modelfield = new eu.eudat.entities.xmlmodels.datasetprofiledefinition.Field();
			//modelfield.fromXml(field);
			AutoCompleteData data = new AutoCompleteData().fromData(modelfield.getData());

			URL url = new URL(data.getUrl()+lookupItem.getSearchTerm());
			HttpURLConnection con = (HttpURLConnection) url.openConnection();
			return ResponseEntity.status(HttpStatus.OK).body(null);
		}
		catch(Exception ex) {
			return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("Serialization issue: "+ex.getMessage());
		}
	} 
}
