package eu.eudat.controllers;

import eu.eudat.models.login.Credentials;
import eu.eudat.models.helpers.responses.ResponseItem;
import eu.eudat.models.login.LoginInfo;
import eu.eudat.models.security.Principal;
import eu.eudat.security.CustomAuthenticationProvider;
import eu.eudat.services.AuthenticationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import javax.transaction.Transactional;

/**
 * Created by ikalyvas on 12/15/2017.
 */
@RestController
@CrossOrigin
@RequestMapping(value = "/auth")
public class Login {

    @Autowired
    private CustomAuthenticationProvider customAuthenticationProvider;

    @Autowired
    private AuthenticationService authenticationService;

    @Transactional
    @RequestMapping(method = RequestMethod.POST, value = { "/externallogin" }, consumes = "application/json", produces="application/json")
    public @ResponseBody ResponseItem<Principal> googleLogin(@RequestBody LoginInfo credentials) {
        try {
            return new ResponseItem<Principal>().payload(customAuthenticationProvider.authenticate(credentials)).status(HttpStatus.OK);

        } catch (Exception ex) {
            ex.printStackTrace();
            return new ResponseItem<Principal>().status(HttpStatus.BAD_REQUEST).message(ex.getMessage());
        }
    }

    @RequestMapping(method = RequestMethod.POST, value = { "/me" }, consumes = "application/json", produces="application/json")
    public @ResponseBody ResponseItem<Principal> authMe(Principal principal) {
        try {
            return new ResponseItem<Principal>().payload(this.authenticationService.Touch(principal.getToken())).status(HttpStatus.OK);

        } catch (Exception ex) {
            ex.printStackTrace();
            return new ResponseItem<Principal>().status(HttpStatus.BAD_REQUEST).message(ex.getMessage());
        }
    }

    @RequestMapping(method = RequestMethod.POST, value = { "/logout" }, consumes = "application/json", produces="application/json")
    public @ResponseBody ResponseItem<Principal> logout(Principal principal) {
        try {
            this.authenticationService.Logout(principal.getToken());
            return new ResponseItem<Principal>().status(HttpStatus.OK);

        } catch (Exception ex) {
            ex.printStackTrace();
            return new ResponseItem<Principal>().status(HttpStatus.BAD_REQUEST).message(ex.getMessage());
        }
    }
}
