package eu.eudat.dao.entities;

import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;

import eu.eudat.dao.databaselayer.service.DatabaseService;
import eu.eudat.models.criteria.ProjectCriteria;
import eu.eudat.queryable.QueryableList;
import org.hibernate.query.Query;

import eu.eudat.dao.JpaDao;
import eu.eudat.entities.Project;
import eu.eudat.entities.responses.IDLabelPair;
import eu.eudat.models.project.ProjectTableRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component("projectDao")
public class ProjectDaoImpl implements ProjectDao {

	@Autowired
	DatabaseService<Project> databaseService;

	@Override
	public QueryableList<Project> getWithCriteria(ProjectCriteria criteria) {
		QueryableList<Project> query = databaseService.getQueryable(Project.class);
		if(criteria.getLike()!=null&&!criteria.getLike().isEmpty())query.where((builder, root) -> builder.like(root.get("label"),"%"+criteria.getLike()+"%"));
		if(criteria.getPeriodEnd()!=null)query.where((builder, root) -> builder.lessThan(root.get("created"),criteria.getPeriodEnd()));
		if(criteria.getPeriodStart()!=null)query.where((builder, root) -> builder.greaterThan(root.get("created"),criteria.getPeriodStart()));
		return query;
	}

	@Override
	public Project createOrUpdate(Project item) {
		return databaseService.createOrUpdate(item,Project.class);
	}

	@Override
	public Project find(UUID id) {
		return databaseService.getQueryable(Project.class).where((builder, root) -> builder.equal((root.get("id")),id)).toList().get(0);
	}

	public Long count(){
		return this.databaseService.count(Project.class);
	}
}
