package eu.eudat.entities;


import java.io.Serializable;
import java.util.Date;
import java.util.Set;
import java.util.UUID;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.Type;

import com.fasterxml.jackson.annotation.JsonIdentityInfo;
import com.fasterxml.jackson.annotation.ObjectIdGenerators;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;


@Entity
@Table(name="\"Project\"")
@JsonIdentityInfo(generator=ObjectIdGenerators.PropertyGenerator.class, property="id")
public class Project implements Serializable,DataEntity<Project> {

	private static final long serialVersionUID = -767048645098311154L;

	public Project () {}
	
	
	@Id
	@GeneratedValue
    @GenericGenerator(name = "uuid2", strategy = "uuid2")
    @Column(name = "\"ID\"", updatable = false, nullable = false, columnDefinition = "BINARY(16)")
	private UUID id;
	
	
	@OneToMany(mappedBy = "project")
	private Set<DMP> dmps;
	
	
	@Column(name = "\"Label\"")
	private String label;
	
	@Column(name = "\"Abbreviation\"")
	private String  abbreviation;

	@Type(type="eu.eudat.typedefinition.XMLType")
	@Column(name = "\"Reference\"", columnDefinition = "xml", nullable = true)
	private String reference;
	
	@Column(name = "\"Uri\"")
	private String uri;
	
	@Type(type="eu.eudat.typedefinition.XMLType")
	@Column(name = "\"Definition\"", columnDefinition = "xml", nullable = true)
	private String definition;
	
	
	@Column(name = "\"StartDate\"", nullable = false)
	private Date startdate = null;
	
	@Column(name = "\"EndDate\"", nullable = false)
	private Date enddate = null;
	
	
	@Column(name = "\"Status\"", nullable = false)
	private Short status; 
	
	
    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "\"CreationUser\"", nullable = true)
	private UserInfo creationUser;
	
	
	@Column(name = "\"Created\"")
	private Date created = null;
	
	@Column(name = "\"Modified\"")
	private Date modified = new Date();
	
	@Column(name = "\"Description\"")
	private String description;
	
	
	public String getDescription() {
		return description;
	}


	public void setDescription(String description) {
		this.description = description;
	}


	public Short getStatus() {
		return status;
	}


	public void setStatus(Short status) {
		this.status = status;
	}


	public Date getCreated() {
		return created;
	}


	public void setCreated(Date created) {
		this.created = created;
	}


	public Date getModified() {
		return modified;
	}


	public void setModified(Date modified) {
		this.modified = modified;
	}
	
	
	public Date getStartdate() {
		return startdate;
	}


	public void setStartdate(Date startdate) {
		this.startdate = startdate;
	}


	public Date getEnddate() {
		return enddate;
	}


	public void setEnddate(Date enddate) {
		this.enddate = enddate;
	}


	public UUID getId() {
		return id;
	}

	public void setId(UUID id) {
		this.id = id;
	}

	public String getLabel() {
		return label;
	}

	public void setLabel(String label) {
		this.label = label;
	}

	public String getAbbreviation() {
		return abbreviation;
	}

	public void setAbbreviation(String abbreviation) {
		this.abbreviation = abbreviation;
	}

	public String getReference() {
		return reference;
	}

	public void setReference(String reference) {
		this.reference = reference;
	}

	public String getUri() {
		return uri;
	}

	public void setUri(String uri) {
		this.uri = uri;
	}

	public String getDefinition() {
		return definition;
	}

	public void setDefinition(String definition) {
		this.definition = definition;
	}


	public Set<DMP> getDmps() {
		return dmps;
	}

	public void setDmps(Set<DMP> dmps) {
		this.dmps = dmps;
	}


	public UserInfo getCreationUser() {
		return creationUser;
	}


	public void setCreationUser(UserInfo creationUser) {
		this.creationUser = creationUser;
	}


	public String toString() {
		try {
			return new ObjectMapper().enable(SerializationFeature.INDENT_OUTPUT).writeValueAsString(this).replace("\"", "&quot;");
		} catch (JsonProcessingException e) {
			return "";
		}
	}

	@Override
	public void update(Project entity) {
		this.description = entity.getDescription();
		this.label = entity.getLabel();
		this.abbreviation = entity.getAbbreviation();
		this.created = entity.getCreated();
		this.definition = entity.getDefinition();
		this.dmps = entity.getDmps();
		this.enddate = entity.getEnddate();
		//this.creationUser = entity.getCreationUser(); //TODO
	}

	@Override
	public Object[] getKeys() {
		return new UUID[]{this.id == null ? null : this.id};
	}
}
