package eu.eudat.entities;

import java.io.Serializable;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import javax.persistence.*;

import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.Type;

import com.fasterxml.jackson.annotation.JsonIdentityInfo;
import com.fasterxml.jackson.annotation.ObjectIdGenerators;


@Entity
@Table(name="\"UserInfo\"")
@JsonIdentityInfo(generator=ObjectIdGenerators.PropertyGenerator.class, property="id")
public class UserInfo implements Serializable,DataEntity<UserInfo>{

	private static final long serialVersionUID = 1225151430484658395L;
	
	@Id
	@GeneratedValue
    @GenericGenerator(name = "uuid2", strategy = "uuid2")
    @Column(name = "id", updatable = false, nullable = false, columnDefinition = "BINARY(16)")
	private UUID id;
	
	
	@Column(name = "email", nullable = false)
	private String email = null;
	
	@Column(name = "authorization_level", nullable = false)
	private Short authorization_level; //0 admin, 1 user
	
	@Column(name = "usertype", nullable = false)
	private Short usertype; // 0 internal, 1 external
	
	@Column(name = "verified_email", nullable = true)
	private Boolean verified_email = null;
	
	@Column(name = "name", nullable = true)
	private String name = null;
	
	
	@Column(name = "created", nullable = false)
	private Date created = null;
	
	
	@Column(name = "lastloggedin", nullable = true)
	private Date lastloggedin = null;

	
	@Type(type="eu.eudat.typedefinition.XMLType")
	@Column(name = "additionalinfo", columnDefinition = "xml", nullable = true)
	private String additionalinfo;

	@OneToMany(fetch = FetchType.LAZY)
    @JoinTable(name="\"UserDMP\"", 
    		joinColumns={@JoinColumn(name="usr", referencedColumnName="id")},
    	    inverseJoinColumns={@JoinColumn(name="dmp", referencedColumnName="\"ID\"")}
    )  
    private Set<DMP> dmps;

	@OneToMany(mappedBy="userInfo",fetch = FetchType.LAZY)
	Set<Credential> credentials = new HashSet<>();
	
	public Set<DMP> getDmps() {
		return dmps;
	}

	public void setDmps(Set<DMP> dmps) {
		this.dmps = dmps;
	}

	public UUID getId() {
		return id;
	}

	public void setId(UUID id) {
		this.id = id;
	}

	public Date getCreated() {
		return created;
	}

	public void setCreated(Date created) {
		this.created = created;
	}

	public Date getLastloggedin() {
		return lastloggedin;
	}

	public void setLastloggedin(Date lastloggedin) {
		this.lastloggedin = lastloggedin;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public Short getAuthorization_level() {
		return authorization_level;
	}

	public void setAuthorization_level(Short authorization_level) {
		this.authorization_level = authorization_level;
	}

	public Short getUsertype() {
		return usertype;
	}

	public void setUsertype(Short usertype) {
		this.usertype = usertype;
	}

	public Boolean getVerified_email() {
		return verified_email;
	}

	public void setVerified_email(Boolean verified_email) {
		this.verified_email = verified_email;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getAdditionalinfo() {
		return additionalinfo;
	}

	public void setAdditionalinfo(String additionalinfo) {
		this.additionalinfo = additionalinfo;
	}

	public Set<Credential> getCredentials() {
		return credentials;
	}

	public void setCredentials(Set<Credential> credentials) {
		this.credentials = credentials;
	}

	@Override
	public void update(UserInfo entity) {

	}

	@Override
	public Object[] getKeys() {
		return new Object[0];
	}
}
