package eu.eudat.managers;

import java.util.List;
import java.util.UUID;

import eu.eudat.dao.entities.*;
import eu.eudat.entities.DMP;
import eu.eudat.entities.Project;
import eu.eudat.entities.UserInfo;
import eu.eudat.models.criteria.DataManagementPlanCriteria;
import eu.eudat.models.criteria.OrganisationCriteria;
import eu.eudat.models.criteria.ProjectCriteria;
import eu.eudat.models.criteria.ResearcherCriteria;
import eu.eudat.models.dmp.DataManagementPlan;
import eu.eudat.models.dmp.DataManagementPlanCriteriaRequest;
import eu.eudat.models.dmp.DataManagementPlanTableRequest;
import eu.eudat.models.helpers.DataTableData;
import eu.eudat.models.listingmodels.DataManagementPlanListingModel;
import eu.eudat.models.security.Principal;
import eu.eudat.queryable.QueryableList;
import eu.eudat.utilities.builders.DomainModelConverter;

public class DataManagementPlanManager {

	public DataTableData<DataManagementPlanListingModel> getPaged(DMPDao dmpsRepository, DataManagementPlanTableRequest dataManagementPlanTableRequest) throws IllegalAccessException, InstantiationException{
		QueryableList<DMP> items = dmpsRepository.getWithCriteria(dataManagementPlanTableRequest.getCriteria());
		QueryableList<DMP> pagedItems = PaginationManager.applyPaging(items,dataManagementPlanTableRequest);
		if(dataManagementPlanTableRequest.getWithHint())pagedItems.withHint("fullyDetailed");
		List<DataManagementPlanListingModel> datamanagementPlans = new DomainModelConverter<eu.eudat.entities.DMP, DataManagementPlanListingModel>().fromDataModel( pagedItems.toList(), DataManagementPlanListingModel.class);
		DataTableData<DataManagementPlanListingModel> dataTable = new DataTableData<DataManagementPlanListingModel>();
		dataTable.setData(datamanagementPlans);
		dataTable.setTotalCount(items.count());
		return dataTable;		
	}
	
	public eu.eudat.models.dmp.DataManagementPlan getSingle(DMPDao dmpsRepository, String id) throws InstantiationException, IllegalAccessException{
		eu.eudat.models.dmp.DataManagementPlan datamanagementPlan = new eu.eudat.models.dmp.DataManagementPlan();
		datamanagementPlan.fromDataModel(dmpsRepository.find(UUID.fromString(id)));
		return datamanagementPlan;
	}

	public List<DataManagementPlan> getWithCriteria(DMPDao dmpsRepository, DataManagementPlanCriteriaRequest dataManagementPlanCriteria) throws IllegalAccessException, InstantiationException{
		QueryableList<DMP> items = dmpsRepository.getWithCriteria(dataManagementPlanCriteria.getCriteria());
		List<eu.eudat.models.dmp.DataManagementPlan> datamanagementPlans = new DomainModelConverter<eu.eudat.entities.DMP, eu.eudat.models.dmp.DataManagementPlan>().fromDataModel( items.toList(), eu.eudat.models.dmp.DataManagementPlan.class);
		return datamanagementPlans;
	}

	public static void createOrUpdate(DMPDao dmpsRepository, ProjectDao projectDao,ResearcherDao researcherRepository, UserInfoDao userInfoDao, OrganisationDao organisationRepository, DataManagementPlan dataManagementPlan, Principal principal){
		DMP newDmp = dataManagementPlan.toDataModel();
		createOrganisationsIfTheyDontExist(newDmp,organisationRepository);
		createResearchersIfTheyDontExist(newDmp,researcherRepository);
		UserInfo user = userInfoDao.read(principal.getId());
		createProjectIfItDoesntExist(newDmp,projectDao,user);
		newDmp.setCreator(user);
		dmpsRepository.createOrUpdate(newDmp);

	}

	private static void createResearchersIfTheyDontExist(DMP newDmp,ResearcherDao researcherRepository){
		if(newDmp.getResearchers()!=null&&!newDmp.getResearchers().isEmpty()){
			for(eu.eudat.entities.Researcher researcher : newDmp.getResearchers()){
				ResearcherCriteria criteria = new ResearcherCriteria();
				criteria.setLike(researcher.getReference());
				List<eu.eudat.entities.Researcher> entries = researcherRepository.listBy(criteria);
				if(entries!=null&&!entries.isEmpty())researcher.setId(entries.get(0).getId());
				else researcher = researcherRepository.create(researcher);
			}
		}
	}

	private static void createOrganisationsIfTheyDontExist(DMP newDmp,OrganisationDao organisationRepository){
		if(newDmp.getOrganisations()!=null&&!newDmp.getOrganisations().isEmpty()){
			for(eu.eudat.entities.Organisation organisation: newDmp.getOrganisations()){
				OrganisationCriteria criteria = new OrganisationCriteria();
				criteria.setLike(organisation.getReference());
				List<eu.eudat.entities.Organisation> entries = organisationRepository.listBy(criteria);
				if(entries!=null&&!entries.isEmpty())organisation.setId(entries.get(0).getId());
				else organisation = organisationRepository.create(organisation);
			}
		}
	}

	private static void createProjectIfItDoesntExist(DMP newDmp,ProjectDao projectDao,UserInfo userInfo){
		if(newDmp.getProject()!=null){
				Project project = newDmp.getProject();
				ProjectCriteria criteria = new ProjectCriteria();
				criteria.setLike(project.getReference());
				List<eu.eudat.entities.Project> entries = projectDao.getWithCriteria(criteria).toList();
				if(entries!=null&&!entries.isEmpty())project.setId(entries.get(0).getId());
				else{
					project.setCreationUser(userInfo);
					projectDao.createOrUpdate(project);
				}
		}
	}
}
