package eu.eudat.models.user.components.datasetprofile;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import eu.eudat.models.components.commons.Multiplicity;
import eu.eudat.models.user.composite.PropertiesModelBuilder;
import eu.eudat.utilities.ModelDefinition;
import eu.eudat.utilities.ViewStyleDefinition;
import eu.eudat.utilities.builders.ModelBuilder;

public class FieldSet implements Comparable,PropertiesModelBuilder, ViewStyleDefinition<eu.eudat.entities.xmlmodels.datasetprofiledefinition.FieldSet>{
	private String id;
	private Integer ordinal;
	private String title;
	private String description;
	private String extendedDescription;
	private Multiplicity multiplicity;
	private List<Field> fields;
	private List<FieldSet> multiplicityItems;
	public List<Field> getFields() {
		Collections.sort(this.fields);
		return fields;
	}

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getExtendedDescription() {
		return extendedDescription;
	}

	public void setExtendedDescription(String extendedDescription) {
		this.extendedDescription = extendedDescription;
	}

	public void setFields(List<Field> fields) {
		this.fields = fields;
	}	

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public int getOrdinal() {
		return ordinal;
	}

	public void setOrdinal(int ordinal) {
		this.ordinal = ordinal;
	}

	public Multiplicity getMultiplicity() {
		return multiplicity;
	}

	public void setMultiplicity(Multiplicity multiplicity) {
		this.multiplicity = multiplicity;
	}
	
	

	public List<FieldSet> getMultiplicityItems() {
		return multiplicityItems;
	}

	public void setMultiplicityItems(List<FieldSet> multiplicityItems) {
		this.multiplicityItems = multiplicityItems;
	}

	public void setOrdinal(Integer ordinal) {
		this.ordinal = ordinal;
	}


	@Override
	public eu.eudat.entities.xmlmodels.datasetprofiledefinition.FieldSet toDatabaseDefinition(eu.eudat.entities.xmlmodels.datasetprofiledefinition.FieldSet item) {
		List<eu.eudat.entities.xmlmodels.datasetprofiledefinition.Field> viewStylefields = new ModelBuilder().toViewStyleDefinition(this.fields, eu.eudat.entities.xmlmodels.datasetprofiledefinition.Field.class);
		item.setFields(viewStylefields);
		item.setId(this.id);
		item.setOrdinal(this.ordinal);
		return item;
	}

	@Override
	public void fromDatabaseDefinition(eu.eudat.entities.xmlmodels.datasetprofiledefinition.FieldSet item) {
		this.fields = new ModelBuilder().fromViewStyleDefinition(item.getFields(), Field.class);		
		this.id = item.getId();
		this.ordinal = item.getOrdinal();
		this.title = item.getTitle();
		this.description = item.getDescription();
		this.extendedDescription = item.getExtendedDescription();
	}

	@Override
	public void fromJsonObject(Map<String, Object> properties) {		
		this.multiplicityItems = new LinkedList<FieldSet>();
		this.fields.forEach(item->{
			item.fromJsonObject(properties);
		});
		
		List<String> compositeKeys =  properties.keySet().stream().filter(keys->keys.startsWith("multiple_"+this.getId())).collect(Collectors.toList());
		for (String key : compositeKeys) {
			this.multiplicityItems.add(this.cloneForMultiplicity(key,compositeKeys.indexOf(key), properties));
		}
	}
	
	private FieldSet cloneForMultiplicity(String key, int index,Map<String, Object> properties){
		FieldSet newFieldSet = new FieldSet();
		newFieldSet.id = key;
		newFieldSet.description = this.description;
		newFieldSet.extendedDescription = this.extendedDescription;
		newFieldSet.title = this.title;

		newFieldSet.ordinal = this.ordinal;
		newFieldSet.fields = new LinkedList();
		this.fields.forEach(item->{
			newFieldSet.fields.add(item.cloneForMultiplicity(key, properties));
		});
		//newFieldSet.fields.forEach(item->item.fromJsonObject(properties));
		return newFieldSet;
	}
	
	private void createMultiplicityFields(List<FieldSet> fieldSet,Field field){
		FieldSet newFieldSet = new FieldSet();
		newFieldSet.fields = new LinkedList<Field>();
		newFieldSet.fields.add(field);
		fieldSet.add(newFieldSet);
	}

	@Override
	public int compareTo(Object o) {
		return this.ordinal.compareTo(((FieldSet)o).getOrdinal());
	}

	@Override
	public void fromJsonObject(Map<String, Object> properties, String path) {
		// TODO Auto-generated method stub
		
	}
	
	
}
