import { DatasetProfileDefinitionModel } from '../models/DatasetProfileDefinitionModel';
import { TranslateService } from '@ngx-translate/core';
import { BaseErrorModel } from '../models/error/BaseErrorModel';
import { ActivatedRoute, Router, Params} from '@angular/router';
import { DatasetService } from '../services/dataset/dataset.service';
import { DatasetWizardModel } from '../models/datasets/DatasetWizardModel';
import { DatasetProfileCriteria } from '../models/criteria/dataset-profile/DatasetProfileCriteria';
import { Component, ViewChild, OnInit, AfterViewInit, ViewEncapsulation } from "@angular/core";
import { DataManagementPlanService } from "../services/data-management-plan/data-management-plan.service";
import { FormGroup, Validators, FormBuilder } from "@angular/forms";
import { AutoCompleteConfiguration } from "../shared/components/autocomplete/AutoCompleteConfiguration";
import { DatasetWizardService } from "../services/dataset-wizard/dataset-wizard.service";
import { DataManagementPlanCriteria } from "../models/criteria/data-management-plan/DataManagementPlanCriteria";
import { DataManagementPlanModel } from "../models/data-managemnt-plans/DataManagementPlanModel";
import { JsonSerializer } from "../utilities/JsonSerializer";
import { Observable } from "rxjs/Observable";
import { RequestItem } from "../models/criteria/RequestItem";
import { MatPaginator, MatSort, MatSnackBar } from "@angular/material";
import { SnackBarNotificationComponent } from "../shared/components/notificaiton/snack-bar-notification.component";

@Component({
  selector: 'app-dataset-wizard-component',
  templateUrl: 'dataset-wizard.component.html',
  styleUrls: ['./dataset-wizard.component.scss'],
  providers: [DatasetWizardService],
  encapsulation: ViewEncapsulation.None
})
export class DatasetWizardComponent implements AfterViewInit {

  dmpAutoCompleteConfiguration: AutoCompleteConfiguration;
  datasetProfileAutoCompleteConfiguration: AutoCompleteConfiguration;
  datasetWizardModel:DatasetWizardModel;
  isNew = true;
  formGroup:FormGroup;
  constructor(
    private datasetWizardService: DatasetWizardService,
    private formBuilder: FormBuilder,
    private datasetService: DatasetService,
    private route: ActivatedRoute,
    public snackBar: MatSnackBar,    
    public router: Router,
    public language: TranslateService,    
  ) {

  }

  isLinear = false;
  firstStepFormGroup: FormGroup;
  secondFormGroup: FormGroup;

  ngOnInit() {
    let dmpRequestItem: RequestItem<DataManagementPlanCriteria> = new RequestItem();
    dmpRequestItem.criteria = new DataManagementPlanCriteria();
    this.dmpAutoCompleteConfiguration = new AutoCompleteConfiguration(this.datasetWizardService.userDmps.bind(this.datasetWizardService), dmpRequestItem);

    let datasetProfileRequestItem: RequestItem<DatasetProfileCriteria> = new RequestItem();
    datasetProfileRequestItem.criteria = new DatasetProfileCriteria();
    this.route.params.subscribe((params: Params) => {
      const itemId = params['id'];
      if (itemId != null) {
        this.isNew = false;
        this.datasetWizardService.getSingle(itemId).map(data => data as DatasetWizardModel)
          .subscribe(data => {
            this.datasetWizardModel = new JsonSerializer<DatasetWizardModel>().fromJSONObject(data, DatasetWizardModel);
            this.formGroup = this.datasetWizardModel.buildForm();
            this.datasetProfileAutoCompleteConfiguration = new AutoCompleteConfiguration(this.datasetWizardService.getAvailableProfiles.bind(this.datasetWizardService), datasetProfileRequestItem);
            this.formGroup.get("dmp").valueChanges.subscribe(change=>{
                this.datasetProfileAutoCompleteConfiguration.requestItem.criteria.id=change.id
              }
            );
          });
      } else {
        this.datasetWizardModel = new DatasetWizardModel();
        setTimeout(() => {
          this.formGroup = this.datasetWizardModel.buildForm();
          this.datasetProfileAutoCompleteConfiguration = new AutoCompleteConfiguration(this.datasetWizardService.getAvailableProfiles.bind(this.datasetWizardService), datasetProfileRequestItem);
          this.formGroup.get("dmp").valueChanges.subscribe(change=>{
              this.datasetProfileAutoCompleteConfiguration.requestItem.criteria.id=change.id
            }
          );
        });
      }
    });
    
    
  }

  ngAfterViewInit() {
    // this.route.params.subscribe((params: Params) => {
    //   const itemId = params['id'];

    //   if (itemId != null) {
    //     this.isNew = false;
    //     this.dataManagementPlanService.getSingle(itemId).map(data => data as DataManagementPlanModel)
    //       .subscribe(data => {
    //         this.dataManagementPlan = new JsonSerializer<DataManagementPlanModel>().fromJSONObject(data, DataManagementPlanModel);
    //         this.formGroup = this.dataManagementPlan.buildForm();
    //       });
    //   } else {
    //     this.dataManagementPlan = new DataManagementPlanModel();
    //     setTimeout(() => {
    //       this.formGroup = this.dataManagementPlan.buildForm();
    //     });
    //   }
    // });
  }

  public cancel(): void {
    //this.router.navigate(['/dataManagementPlans']);
  }


  formSubmit(): void {
    //this.touchAllFormFields(this.formGroup);
    //if(this.isNew)return;
    if (!this.isFormValid()) { return; }
    this.onSubmit();
  }

  public isFormValid() {
    return this.formGroup.valid;
  }

  onSubmit(): void {
    this.datasetWizardService.createDataset(this.formGroup.value).subscribe(
      complete => {
        this.datasetWizardService.getSingle(complete.id).subscribe(
          result=>{
            this.datasetWizardModel = new JsonSerializer<DatasetWizardModel>().fromJSONObject(result,DatasetWizardModel);
          }
        )
        this.onCallbackSuccess()
      },
      error => this.onCallbackError(error)
    );
  }

  onCallbackSuccess(): void {
    this.snackBar.openFromComponent(SnackBarNotificationComponent, {
      data: { message: this.isNew ? 'GENERAL.SNACK-BAR.SUCCESSFUL-CREATION' : 'GENERAL.SNACK-BAR.SUCCESSFUL-UPDATE', language: this.language },
      duration: 3000,
      extraClasses: ['snackbar-success']
    })
  }

  onCallbackError(error: any) {
    this.setErrorModel(error.error);
    //this.validateAllFormFields(this.formGroup);
  }

  public setErrorModel(errorModel: BaseErrorModel) {
    Object.keys(errorModel).forEach(item => {
      (<any>this.datasetWizardModel.errorModel)[item] = (<any>errorModel)[item];
    })
  }

}