import { DatasetProfileDefinitionModel } from '../DatasetProfileDefinitionModel';
import { DatasetProfileModel } from '../datasetprofile/DatasetProfileModel';
import { DataManagementPlanModel } from '../data-managemnt-plans/DataManagementPlanModel';
import { Serializable } from "../Serializable";
import { ValidationContext } from "../../utilities/validators/ValidationContext";
import { FormGroup, FormBuilder, FormControl, Validators } from "@angular/forms";
import { BackendErrorValidator } from "../../utilities/validators/BackendErrorValidator";
import { BaseErrorModel } from "../error/BaseErrorModel";
import { ExternalSourcesItemModel } from "../external-sources/ExternalSourcesItemModel";
import { ServiceModel } from "../services/ServiceModel";
import { JsonSerializer } from "../../utilities/JsonSerializer";
import { RegisterModel } from "../registers/RegisterModel";
import { DataRepositoryModel } from "../dataRepositories/DataRepositoryModel";

export class DatasetWizardModel implements Serializable<DatasetWizardModel> {
	public id: String;
	public label: String;
	public profile: DatasetProfileModel = new DatasetProfileModel;
	public uri: String;
	public status: String;
	public description: String;
	public services: ServiceModel[] = [];
	public registries: RegisterModel[] = [];
	public dataRepositories: DataRepositoryModel[] = [];
	public dmp:DataManagementPlanModel = new DataManagementPlanModel();
	public datasetProfileDefinition = new DatasetProfileDefinitionModel();
	public errorModel: BaseErrorModel = new BaseErrorModel();

	fromJSONObject(item: any): DatasetWizardModel {
		this.id = item.id;
		this.label = item.label;
		this.profile = item.profile;
		this.uri = item.uri;
		this.status = item.status;
		this.description = item.description;
		this.services = new JsonSerializer<ServiceModel>().fromJSONArray(item.services, ServiceModel);
		this.registries = new JsonSerializer<RegisterModel>().fromJSONArray(item.registries, RegisterModel);
		this.dataRepositories = new JsonSerializer<DataRepositoryModel>().fromJSONArray(item.dataRepositories, DataRepositoryModel);
        this.dmp = new JsonSerializer<DataManagementPlanModel>().fromJSONObject(item.dmp, DataManagementPlanModel);
		this.profile = new JsonSerializer<DatasetWizardModel>().fromJSONObject(item.profile, DatasetWizardModel);
		this.datasetProfileDefinition = new JsonSerializer<DatasetProfileDefinitionModel>().fromJSONObject(item.datasetProfileDefinition, DatasetProfileDefinitionModel);
		return this;
	}

	buildForm(context: ValidationContext = null, disabled: boolean = false): FormGroup {
		if (context == null) { context = this.createValidationContext(); }

		const formGroup = new FormBuilder().group({
			label: [{ value: this.label, disabled: disabled }, context.getValidation('label').validators],
			uri: [{ value: this.uri, disabled: disabled }, context.getValidation('uri').validators],
			status: [{ value: this.status, disabled: disabled }, context.getValidation('status').validators],
			description: [{ value: this.description, disabled: disabled }, context.getValidation('description').validators],
			services: [{ value: this.services, disabled: disabled }, context.getValidation('services').validators],
			registries: [{ value: this.registries, disabled: disabled }, context.getValidation('registries').validators],
			dataRepositories: [{ value: this.dataRepositories, disabled: disabled }, context.getValidation('dataRepositories').validators],
			dmp: [{ value: this.dmp, disabled: disabled }, context.getValidation('dmp').validators],
			profile: [{ value: this.profile, disabled: disabled }, context.getValidation('profile').validators],
			
		});		
		return formGroup;
	}

	createValidationContext(): ValidationContext {
		const baseContext: ValidationContext = new ValidationContext();
		baseContext.validation.push({ key: 'label', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'label')] });
		baseContext.validation.push({ key: 'profile', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'profile')] });
		baseContext.validation.push({ key: 'uri', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'uri')] });
		baseContext.validation.push({ key: 'status', validators: [ BackendErrorValidator(this.errorModel, 'status')] });
		baseContext.validation.push({ key: 'description', validators: [Validators.required, BackendErrorValidator(this.errorModel, 'description')] });
		baseContext.validation.push({ key: 'services', validators: [ BackendErrorValidator(this.errorModel, 'services')] });
		baseContext.validation.push({ key: 'registries', validators: [ BackendErrorValidator(this.errorModel, 'registries')] });
		baseContext.validation.push({ key: 'dataRepositories', validators: [BackendErrorValidator(this.errorModel, 'dataRepositories')] });
		baseContext.validation.push({ key: 'dmp', validators: [BackendErrorValidator(this.errorModel, 'dmp')] });   //TODO
		return baseContext;
	}
}