import { Injectable } from '@angular/core';
import { HttpHeaders } from '@angular/common/http';
import { HttpClient } from '@angular/common/http';
import { HostConfiguration } from '../../app.constants';
import { Principal } from '../../models/login/Principal';
import { Credential } from '../../models/login/Credential';
import { Observable } from 'rxjs/Rx';
import { MatSnackBar } from '@angular/material';
import { TranslateService } from '@ngx-translate/core';
import { Router } from '@angular/router';
import { JsonSerializer } from '../../utilities/JsonSerializer';
import { LoginInfo } from '../../models/login/LoginInfo';
import { SnackBarNotificationComponent } from '../../shared/components/notificaiton/snack-bar-notification.component';

@Injectable()
export class AuthService {
	private actionUrl: string;
	private headers: HttpHeaders;

	constructor(
		private http: HttpClient,
		public snackBar: MatSnackBar,
		public language: TranslateService,
		public router: Router
	) {

		this.actionUrl = HostConfiguration.Server + 'auth/';

		this.headers = new HttpHeaders();
		this.headers = this.headers.set('Content-Type', 'application/json');
		this.headers = this.headers.set('Accept', 'application/json');
	}

	private clear(): void {
		localStorage.removeItem('principal');
	}

	public current(principal?: Principal): Principal {
		if (principal) {
			localStorage.setItem('principal', JSON.stringify(principal));
			return principal;
		}
		const principalJson = localStorage.getItem('principal');
		if (!principalJson) { return null; }
		const principalObj = JSON.parse(principalJson) as Principal;
		return principalObj;
	}

	//public login(credential: Credential): Observable<Principal> {
	//    const url = this.actionUrl + 'login';

	//    return this.http.post(url, credential, { headers: this.headers })
	//        .map((res: Response) => {
	//            let principal = this.current(new JsonSerializer<Principal>().fromJSONObject(res, Principal));
	//            //this.loginContextSubject.next(true);
	//            return principal;
	//        })
	//        .catch((error: any) => {
	//            //this.loginContextSubject.next(false);
	//            return Observable.throw(error);
	//        });
	//}

	public login(loginInfo: LoginInfo): Observable<Principal> {
		const url = this.actionUrl + 'externallogin';

		return this.http.post(url, loginInfo, { headers: this.headers })
			.map((res: any) => {
				const principal = this.current(new JsonSerializer<Principal>().fromJSONObject(res.payload, Principal));
				//this.loginContextSubject.next(true);
				return principal;
			})
			.catch((error: any) => {
				//this.loginContextSubject.next(false);
				return Observable.throw(error);
			});
	}

	public logout(): void {
		const url = this.actionUrl + 'logout';
		const principal = this.current();
		this.clear();

		if (!principal) { return; }
		let headers = this.headers;
		headers = headers.set('AuthToken', principal.token)
		this.http.post(url, null, { headers: headers }).subscribe(
			res => this.onLogOutSuccess(res),
			error => this.onLogOutError(error)
		)
	}

	public me(): Observable<Principal> {
		const url = this.actionUrl + '/me';
		const principal = this.current();
		if (!principal) {
			this.clear();
			return Observable.of<Principal>();
		}
		let headers = this.headers;
		headers = headers.set('AuthToken', principal.token)
		return this.http.post(url, null, { headers: headers })
			.map((r: Response) => {
				const princ = this.current(new JsonSerializer<Principal>().fromJSONObject(r.json(), Principal));
				//this.loginContextSubject.next(true);
				return princ;
			})
			.catch((error: any) => {
				console.warn('could not retrieve me info:\n', error);
				this.clear();
				const princ = this.current();
				this.router.navigate(['/login']);
				return Observable.of<Principal>(princ);
			});
	}

	public onLogOutSuccess(logoutMessage: any) {
		this.snackBar.openFromComponent(SnackBarNotificationComponent, {
			data: { message: 'GENERAL.SNACK-BAR.SUCCESSFUL-LOGOUT', language: this.language },
			duration: 3000,
			extraClasses: ['snackbar-success']
		})
		this.router.navigate(['/login']);
	}

	public onLogOutError(errorMessage: string) {
		console.log(errorMessage);
		this.snackBar.openFromComponent(SnackBarNotificationComponent, {
			data: { message: 'GENERAL.SNACK-BAR.UNSUCCESSFUL-LOGOUT', language: this.language },
			duration: 3000,
			extraClasses: ['snackbar-warning']
		})
		this.router.navigate(['/login']);
	}
}
