import 'rxjs/add/operator/map';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { HostConfiguration } from './../../app.constants';
import { BaseHttpService } from '../../utilities/cite-http-service-module/base-http.service';
import { Observable } from 'rxjs/Observable';
import { DataTableRequest } from '../../models/data-table/DataTableRequest';
import { DataTableData } from '../../models/data-table/DataTableData';
import { ProjectListingModel } from '../../models/projects/ProjectListingModel';
import { ProjectModel } from '../../models/projects/ProjectModel';
import { BaseHttpResponseModel } from '../../models/http/BaseHttpResponseModel';
import { ProjectCriteria } from '../../models/criteria/project/ProjectCriteria';
import { RequestItem } from '../../models/criteria/RequestItem';


@Injectable()
export class ProjectService {

    private actionUrl: string;
    private headers: HttpHeaders;

    constructor(private http: BaseHttpService) {

        this.actionUrl = HostConfiguration.Server + 'projects/';

        this.headers = new HttpHeaders();
        this.headers = this.headers.set('Content-Type', 'application/json');
        this.headers = this.headers.set('Accept', 'application/json');
    }

    getPaged(dataTableRequest: DataTableRequest<ProjectCriteria>): Observable<DataTableData<ProjectListingModel>> {
        return this.http.post<DataTableData<ProjectListingModel>>(this.actionUrl + 'getPaged', dataTableRequest, { headers: this.headers });
    }

    get(requestItem: RequestItem<ProjectCriteria>): Observable<ProjectModel[]> {
        return this.http.post<ProjectModel[]>(this.actionUrl + 'get', requestItem, { headers: this.headers });
    }

    getWithExternal(requestItem: RequestItem<ProjectCriteria>): Observable<ProjectModel[]> {
        return this.http.post<ProjectModel[]>(this.actionUrl + 'getWithExternal', requestItem, { headers: this.headers });
    }

    getSingle(id: string): Observable<ProjectModel> {
        return this.http.get<ProjectModel>(this.actionUrl + 'getSingle/' + id, { headers: this.headers });
    }

    createProject(projectModel: ProjectModel): Observable<ProjectModel> {
        return this.http.post<ProjectModel>(this.actionUrl + 'createOrUpdate', projectModel, { headers: this.headers });
    }

    inactivate(id: String): Observable<ProjectModel> {
        return this.http.delete<ProjectModel>(this.actionUrl + 'inactivate/' + id, { headers: this.headers });
    }
}
