package dmp.javascript.tests;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.FileNotFoundException;
import java.io.FileReader;

import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import dmp.javascript.JSEngine;

public class TestEngine {

	
	private static JSEngine jsEngine;

	@BeforeClass
	public static void init() {
		
	}

	@Before
	public void beforeEachTest() {
		jsEngine = new JSEngine();
	}

	@After
	public void afterEachTest() {
		
	}

	@Test
	public void testFunction1() {
		try {
			jsEngine.addScript("function f(){return 1;}");
			Object obj = jsEngine.executeFunction("f");
			assertEquals(1, obj);
		}
		catch(Exception ex) {
			fail("Should not have thrown an exception");
		}
	}

	@Test
	public void testFunction2() {
		try {
			jsEngine.addScript("function mergeStr(p1,p2){return p1+p2;}");
			Object obj = jsEngine.executeFunction("mergeStr", "aaa", "bbb");
			assertEquals("aaabbb", obj);
		}
		catch(Exception ex) {
			fail("Should not have thrown an exception");
		}
	}
	
	@Test
	public void testFunction3() {
		try {
			jsEngine.addScript("var a = 5;");
			jsEngine.addScript("function f(){return a;}");
			Object obj = jsEngine.executeFunction("f");
			assertEquals(5, obj);
		}
		catch(Exception ex) {
			ex.printStackTrace();
			fail("Should not have thrown an exception");
		}
	}
	
	@Test
	public void testSubFunction() {
		try {
			jsEngine.addScript("function subf(){return \"sub\";}");
			jsEngine.addScript("function f(p){return subf()+p;}");
			Object obj = jsEngine.executeFunction("f", "aaa");
			assertEquals("subaaa", obj);
		}
		catch(Exception ex) {
			fail("Should not have thrown an exception");
		}
	}
	
	@Test(expected = FileNotFoundException.class)
	public void testReaderException() throws Exception {
		jsEngine.addScript(new FileReader("/sdghsfh/sdgafg/f.f"));
	}
	
}
